/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.model;

import org.netbeans.api.project.Project;
import org.openide.util.Lookup;

/**
 *
 * @author johan
 */
public abstract class AbstractEntityManager<E extends Entity> implements EntityManager<E> {
    private static final EntityCache CACHE = Lookup.getDefault().lookup(EntityCache.class);
    
    @Override
    public boolean supports(ItemIdType itemIdType) {
        if (getSupportedTypes() == null) {
            return false;
        }
        return getSupportedTypes().contains(itemIdType);
    }

    /**
     * Find an entity instance by its URI.
     * 
     * @param <E>
     * @param uri
     * @return the entity instance or null when not found
     */
    @Override
    // FIXME: doesn't work for nested entities anymore now that the generic type has been moved from method to class level.
    public E find(AgroURI uri) {
        
        E entity = (E) CACHE.get(uri);
        
        if (entity == null) {
            // load() implementations are responsible for adding the entity to the cache
            entity = load(uri);
        }
                
        return entity;
    }
    
    /**
     * add an entity instance to the cache.
     * override in subclasses to handle cascading to nested/embedded entities.
     * 
     * @param entity 
     */
    protected void addToCache(Entity entity) {
        CACHE.add(entity);
    }
    
    /**
     * remove an entity instance from the cache.
     * override in subclasses to handle cascading to nested/embedded entities.
     * 
     * @param entity 
     */
    protected void removeFromCache(Entity entity) {
        CACHE.remove(entity);
    }
    
    /**
     * Finds or creates a cached instance for the input entity's URI.
     * 
     * helps ensure that a single entity instance is used for each URI key.
     * 
     * @param entity
     * @return either the already cached instance for the input entity's URI, 
     *          or, if none was found, the input entity itself.
     *          Should never return null for non-null input.
     */
    protected E getCachedInstanceFor(E entity) {
        E cachedInstance = (E) CACHE.get(entity.getURI());
        if (cachedInstance == null) {
            addToCache(entity);
            cachedInstance = entity;
        }
        return cachedInstance;
    }
       
    protected E load(AgroURI uri) {
        Project project = EnterpriseIdProvider.ProjectFinder.find(uri);
        return load(uri, project);
    }
    
    protected abstract E load(AgroURI uri, Project project);
    
//    public void save(Entity entity) {
//        Project project = EnterpriseIdProvider.ProjectFinder.find(entity.getURI());
//        save(entity, project);
//    }
//
//    protected abstract void save(Entity entity, Project project);

}
