/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.model;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;

/**
 * Address details conform EDI-Teelt.
 * 
 * @author johan & Merijn Zengers
 */
public class Address implements Serializable{
    public static final String PROP_STREET_NAME = "streetName";
    private String streetName; // an..30
    public static final String PROP_HOUSE_NUMBER = "houseNumber";
    private Integer houseNumber; // n..6
    public static final String PROP_POSTAL_CODE = "postalCode";
    private String postalCode; // an..6
    public static final String PROP_CITY = "city";
    private String city; // an..30
    public static final String PROP_COUNTRY = "country";
    private Country country; // an..3: 3 character ISO code
    public static final String PROP_ADDITION = "addition";
    private String addition;
    
    private transient PropertyChangeSupport propertyChangeSupport;

    public Address() {
    }

    public Address(String streetName, int houseNumber, String postalCode, String city, Country country) {
        this.streetName = streetName;
        this.houseNumber = houseNumber;
        this.postalCode = postalCode;
        this.city = city;
        this.country = country;
    }
    
    private PropertyChangeSupport getPropertyChangeSupport(){
        if(propertyChangeSupport == null){
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }
    
    /**
     * Add a {@link PropertyChangeListener} to this address
     * @param listener 
     */
    public void addPropertyChangeListener(PropertyChangeListener listener){
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }
    
    /**
     * Add a {@link PropertyChangeListener} to this address for a specific property
     * @param prop
     * @param listener 
     */
    public void addPropertyChangeListener(String prop, PropertyChangeListener listener){
        getPropertyChangeSupport().addPropertyChangeListener(prop, listener);
    }
    
    /**
     * remove a {@link PropertyChangeListener} from this address
     * @param listener 
     */
    public void removePropertyChangeListener(PropertyChangeListener listener){
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }
    
    /**
     *  remove a {@link PropertyChangeListener} from this address for a specific property
     * @param property
     * @param listener 
     */
    public void removePropertyChangeListener(String property, PropertyChangeListener listener){
        getPropertyChangeSupport().removePropertyChangeListener(property, listener);
    }

    /**
     * Get the City of this address
     * @return 
     */
    public String getCity() {
        return city;
    }

    /**
     * Set the city of this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param city 
     */
    public void setCity(String city) {
        String old = this.city;
        this.city = city;
        getPropertyChangeSupport().firePropertyChange(PROP_CITY, old, city);
    }

    /**
     * Get the {@link Country} of this address
     * @return 
     */
    public Country getCountry() {
        return country;
    }

    /**
     * Set the {@link Country} of this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param country 
     */
    public void setCountry(Country country) {
        Country old = this.country;
        this.country = country;
        getPropertyChangeSupport().firePropertyChange(PROP_COUNTRY, old, country);
    }

    /**
     * Get the house number of this address
     * @return 
     */
    public Integer getHouseNumber() {
        return houseNumber;
    }

    /**
     * Set the house number for this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param houseNumber 
     */
    public void setHouseNumber(Integer houseNumber) {
        Integer old = this.houseNumber;
        this.houseNumber = houseNumber;
        getPropertyChangeSupport().firePropertyChange(PROP_HOUSE_NUMBER, old, houseNumber);
    }

    /**
     * Get the postal code of this address 
     * @return 
     */
    public String getPostalCode() {
        return postalCode;
    }

    /**
     * Set the postal code for this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param postalCode 
     */
    public void setPostalCode(String postalCode) {
        String old = this.postalCode;
        this.postalCode = postalCode;
        getPropertyChangeSupport().firePropertyChange(PROP_POSTAL_CODE, old, postalCode);
    }

    /**
     * Get the street name of this address
     * @return 
     */
    public String getStreetName() {
        return streetName;
    }

    /**
     * Set the street name for this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param streetName 
     */
    public void setStreetName(String streetName) {
        String old = this.streetName;
        this.streetName = streetName;
        getPropertyChangeSupport().firePropertyChange(PROP_STREET_NAME, old, streetName);
    }

    /**
     * Get the addition for this address
     *
     * @return 
     */    
    public String getAddition() {
        return addition;
    }

    /**
     * Set the addition for this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param addition 
     */
    public void setAddition(String addition) {
        String old = this.addition;
        this.addition = addition;
        getPropertyChangeSupport().firePropertyChange(PROP_ADDITION, old, addition);
    }
    
    

}
