/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.model;

import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;

/**
 *
 * <p>
 * For testing purposes following constructor can be used:
 * 
 * new AgroURI("OTHER/1/TSK/2")
 * 
 * where TSK can be replaced by any value from the enum {@link ItemIdType}
 * </p>
 * 
 * 
 * @author Johan Reitzema
 */
public class AgroURI implements Serializable, EnterpriseIdProvider {
    public static final String DEFAULT_ISSUING_AGENCY = "AGRONL";
    
    private URI uri;
    private transient String[] parts;
    
    public AgroURI(String identifier) {
        assert identifier != null;
        try {
            this.uri = new URI(identifier);
        } catch (URISyntaxException ex) {
            throw new IllegalArgumentException(ex);
        }
        // TODO: validate format, move to factory.
    }
    
    private String[] getParts() {
        if (parts == null) {
            parts = uri.getPath().split("/");
            assert parts.length == 4;            
        }
        return parts;
    }
    
    /**
     * extract the issuing agency from the URI.
     * for the Netherlands this will always be AGRONL.
     * 
     * @return 
     */
    @Override
    public IssuingAgency getIssuingAgency() {
        return IssuingAgency.valueOf(getParts()[0]);
    }
    
    /**
     * extract the enterprise id from the URI.
     * for AGRONL this will be the KvK-number.
     * 
     * @return 
     */
    @Override
    public String getEnterpriseId() {
        return getParts()[1];
    }
    
    /**
     * extract the entity type from the URI.
     * 
     * @return 
     */
    public ItemIdType getItemIdType() {
        return ItemIdType.valueOf(getParts()[2]);
    }
    
    /**
     * extract the entity id from the URI.
     * 
     * @return 
     */
    public String getItemIdNumber() {
        return getParts()[3];
    }
    
    @Override
    public String toString() {
        return uri.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (obj instanceof AgroURI) {
            return uri.equals(((AgroURI)obj).uri);
        }
        return false;
    }

    @Override
    public int hashCode() {
        return uri.hashCode();
    }
    
}
