/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.model;

import java.util.HashMap;
import java.util.regex.Pattern;
import org.openide.util.NbBundle;

/**
 * <p>Validator for postal codes</p>
 *
 * Is able to validate postal for a country
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({"PostalCodeValidator no valid postal code=Invalid postal code for {0}"})
public class PostalCodeValidator {

    private HashMap<Country, String> postalcodePatterns = new HashMap<>();

    public PostalCodeValidator() {
        postalcodePatterns.put(Country.NL, "[0-9]{4}s*[a-zA-Z]{2}$");
        postalcodePatterns.put(Country.JP, "\\d{3}-\\d{4}$");
        postalcodePatterns.put(Country.US, "^\\d{5}\\p{Punct}?\\s?(?:\\d{4})?$");
    }

    /**
     * Validate the postal code Checks if there is a validator available for
     * this country<br/> If there is no validator will not validate
     *
     * @param country
     * @throws validationException when validation fails
     */
    public void validate(Country country, String toValidate) throws ValidationException {
        String pattern = postalcodePatterns.get(country);
        if (pattern != null && !pattern.isEmpty()) {
            Pattern postalCodePattern = Pattern.compile(pattern);
            if (!postalCodePattern.matcher(toValidate).find()) {
                throw new ValidationException(Bundle.PostalCodeValidator_no_valid_postal_code(country.getName()));
            }
        }
    }

    /**
     * Exception to be thrown when validation fails
     *
     * @author merijn
     */
    public class ValidationException extends Exception {

        public ValidationException(String message, Throwable cause, boolean enableSuppression, boolean writableStackTrace) {
            super(message, cause, enableSuppression, writableStackTrace);
        }

        public ValidationException(Throwable cause) {
            super(cause);
        }

        public ValidationException(String message, Throwable cause) {
            super(message, cause);
        }

        public ValidationException(String message) {
            super(message);
        }

        public ValidationException() {
        }
    }
}
